@@ -32,7 +32,6 @@
       INTEGER,PARAMETER :: NNY     = 2
 
       INTEGER(8),PARAMETER :: SIZEOF_REAL4 = 4
-      INTEGER(8),PARAMETER :: INT32_MASK   = INT(Z'00000000FFFFFFFF', 8)
 
       ! Data Fitting task descriptor
       TYPE (DF_TASK) task
@@ -116,14 +115,6 @@
       loc_y  = loc(y)
       loc_y1 = loc_y + nx * SIZEOF_REAL4
 
-      if (BIT_SIZE(loc(y)) == 32) then
-      !***** Adjust pointer values on IA32 architecture *****
-        loc_y = IAND(loc_y,INT32_MASK)
-        y_ret = IAND(y_ret,INT32_MASK)
-        loc_y1 = IAND(loc_y1,INT32_MASK)
-        y1_ret = IAND(y1_ret,INT32_MASK)
-      end if
-
       if ( loc_y /= y_ret ) then
         errnums = errnums + 1
       end if
