@@ -1,7 +1,7 @@
 #!/bin/sh
 #shellcheck shell=sh
 #===============================================================================
-# Copyright 2003-2022 Intel Corporation.
+# Copyright 2003-2024 Intel Corporation.
 #
 # This software and the related documents are Intel copyrighted  materials,  and
 # your use of  them is  governed by the  express license  under which  they were
@@ -14,6 +14,24 @@
 # License.
 #===============================================================================
 
+# ############################################################################
+#
+# Intel(R) oneAPI Math Kernel Library (oneMKL) vars.sh
+# Syntax:
+#
+#   source vars.sh [mod [<MKL_interface>]]
+#
+#     mod (optional) - Set path to oneMKL F95 modules
+#
+#     <MKL_interface> (optional) - oneMKL programming interface for the
+#                                  "mod" option to set path
+#         lp64         : 4 bytes integer (default)
+#         ilp64        : 8 bytes integer
+#
+# If the arguments to the sourced script are ignored (consult docs for
+# your shell), alternatively you can use environment variables MKLVARS_MOD
+# to pass "mod" and MKLVARS_INTERFACE to pass <MKL_interface>.
+#
 # ############################################################################
 
 # Get absolute path to this script.
@@ -89,31 +107,50 @@ if [ "$_vars_this_script_name" = "$(_vars_get_proc_name "$0")" ] ; then
   return 255 2>/dev/null || exit 255
 fi
 
+# ############################################################################
+
+# Prepend path segment(s) to path-like env vars (PATH, CPATH, etc.).
+
+# prepend_path() avoids dangling ":" that affects some env vars (PATH and CPATH)
+# PATH > https://www.gnu.org/software/libc/manual/html_node/Standard-Environment.html
+
+# Usage:
+#   env_var=$(prepend_path "$prepend_to_var" "$existing_env_var")
+#   export env_var
+#
+# Inputs:
+#   $1 == path segment to be prepended to $2
+#   $2 == value of existing path-like environment variable
+
+prepend_path() (
+  path_to_add="$1"
+  path_is_now="$2"
+
+  if [ "" = "${path_is_now}" ] ; then   # avoid dangling ":"
+    printf "%s" "${path_to_add}"
+  else
+    printf "%s" "${path_to_add}:${path_is_now}"
+  fi
+)
 
 # ############################################################################
 
 mkl_help() {
-    __mkl_help=1
     echo ""
-    echo "oneAPI MKL vars.sh syntax:"
-    echo "  source $__mkl_tmp_SCRIPT_NAME [<arch>] [<MKL_interface>] [mod]"
+    echo "oneMKL ${__mkl_tmp_SCRIPT_NAME} syntax:"
     echo ""
-    echo "   <arch> (optional) - oneAPI MKL architecture: "
-    echo "       ia32         : Setup for IA-32 architecture"
-    echo "       intel64      : Setup for Intel(R) 64 architecture (default)"
+    echo "  source ${__mkl_tmp_SCRIPT_NAME} [mod [<MKL_interface>]]"
     echo ""
-    echo "   <MKL_interface> (optional) - oneAPI MKL programming interface for intel64"
-    echo "                                Not applicable without \"mod\""
-    echo "       lp64         : 4 bytes integer (default)"
-    echo "       ilp64        : 8 bytes integer"
+    echo "    mod (optional) - Set path to oneMKL F95 modules"
     echo ""
-    echo "   mod (optional) - set path to oneAPI MKL F95 modules"
+    echo "    <MKL_interface> (optional) - oneMKL programming interface for the"
+    echo "                                 \"mod\" option to set path"
+    echo "        lp64         : 4 bytes integer (default)"
+    echo "        ilp64        : 8 bytes integer"
     echo ""
     echo "If the arguments to the sourced script are ignored (consult docs for"
-    echo "your shell) the alternative way to specify target is environment"
-    echo "variables COMPILERVARS_ARCHITECTURE or MKLVARS_ARCHITECTURE to pass"
-    echo "<arch> to the script, MKLVARS_INTERFACE to pass <MKL_interface> and"
-    echo "MKLVARS_MOD to pass \"mod\""
+    echo "your shell), alternatively you can use environment variables MKLVARS_MOD"
+    echo "to pass \"mod\" and MKLVARS_INTERFACE to pass <MKL_interface>."
     echo ""
 }
 
@@ -182,89 +219,66 @@ fi
 
 # ############################################################################
 
-# Setup environment.
+__mkl_tmp_SCRIPT_NAME=$(basename -- "${vars_script_name:-}")
+__mkl_tmp_SCRIPT_DIR=$(get_script_path "${vars_script_name:-}")
 
-my_script_name=$(basename -- "${vars_script_name:-}")
-my_script_path=$(get_script_path "${vars_script_name:-}")
-MKLROOT=$(dirname -- "${my_script_path}") ; export MKLROOT
-
-__mkl_tmp_SCRIPT_NAME="$my_script_name"
+# Constants
 __mkl_tmp_MOD_NAME=mod
-__mkl_tmp_TARGET_ARCH=intel64
-__mkl_tmp_LP64_ILP64=
-__mkl_tmp_MOD=
-__mkl_tmp_MKLVARS_VERBOSE=
-__mkl_help=
 
-if [ -z "$1" ] ; then
-    if [ -n "$MKLVARS_ARCHITECTURE" ] ; then
-        __mkl_tmp_TARGET_ARCH="$MKLVARS_ARCHITECTURE"
-    elif [ -n "$COMPILERVARS_ARCHITECTURE" ] ; then
-        __mkl_tmp_TARGET_ARCH="$COMPILERVARS_ARCHITECTURE"
-    fi
-    if [ "${__mkl_tmp_TARGET_ARCH}" != "ia32" ] && [ "${__mkl_tmp_TARGET_ARCH}" != "intel64" ] ; then
-        __mkl_tmp_TARGET_ARCH=intel64
-    fi
-    if [ -n "$MKLVARS_INTERFACE" ] ; then
-        __mkl_tmp_LP64_ILP64="$MKLVARS_INTERFACE"
-        if [ "${__mkl_tmp_LP64_ILP64}" != "lp64" ] && [ "${__mkl_tmp_LP64_ILP64}" != "ilp64" ] ; then
-            __mkl_tmp_LP64_ILP64=
+# Defaults
+__mkl_tmp_MOD=
+__mkl_tmp_LP64_ILP64=lp64
+__mkl_tmp_MKLVARS_VERBOSE=
+
+# Parse arguments
+if [ -z "${1:-}" ] ; then
+    # from environment variables
+    if [ -n "${MKLVARS_MOD:-}" ] ; then
+        __mkl_tmp_MOD="$MKLVARS_MOD"
+        if [ "${__mkl_tmp_MOD}" != "mod" ] ; then
+            >&2 echo "   ERROR: Invalid MKLVARS_MOD value \"${__mkl_tmp_MOD}\". The only valid value is \"mod\"."
+            return 254 2>/dev/null || exit 254
+        fi
+        if [ -n "${MKLVARS_INTERFACE:-}" ] ; then
+            __mkl_tmp_LP64_ILP64="$MKLVARS_INTERFACE"
+            if [ "${__mkl_tmp_LP64_ILP64}" != "lp64" ] && [ "${__mkl_tmp_LP64_ILP64}" != "ilp64" ] ; then
+                >&2 echo "   ERROR: Invalid MKLVARS_INTERFACE value \"${__mkl_tmp_LP64_ILP64}\". The valid values are \"lp64\" and \"ilp64\"."
+                return 254 2>/dev/null || exit 254
+            fi
         fi
     fi
-    if [ -n "$MKLVARS_MOD" ] ; then
-        __mkl_tmp_MOD="$MKLVARS_MOD"
-    fi
-    if [ -n "$MKLVARS_VERBOSE" ] ; then
-        __mkl_tmp_MKLVARS_VERBOSE="$MKLVARS_VERBOSE"
-    fi
 else
+    # from command line arguments
     while [ -n "${1:-}" ]; do
-        if   [ "$1" = "ia32" ]        ; then __mkl_tmp_TARGET_ARCH=ia32;
-        elif [ "$1" = "intel64" ]     ; then __mkl_tmp_TARGET_ARCH=intel64;
-        elif [ "$1" = "lp64" ]        ; then __mkl_tmp_LP64_ILP64=lp64;
-        elif [ "$1" = "ilp64" ]       ; then __mkl_tmp_LP64_ILP64=ilp64;
-        elif [ "$1" = "${__mkl_tmp_MOD_NAME}" ] ; then __mkl_tmp_MOD=${__mkl_tmp_MOD_NAME};
-        elif [ "$1" = "verbose" ]     ; then __mkl_tmp_MKLVARS_VERBOSE=verbose;
-        elif [ "$1" = "help" ]        ; then mkl_help;
+        if   [ "$1" = "${__mkl_tmp_MOD_NAME}" ] ; then __mkl_tmp_MOD=${__mkl_tmp_MOD_NAME};
+        elif [ "$1" = "lp64" ]                  ; then __mkl_tmp_LP64_ILP64=lp64;
+        elif [ "$1" = "ilp64" ]                 ; then __mkl_tmp_LP64_ILP64=ilp64;
+        elif [ "$1" = "verbose" ]               ; then __mkl_tmp_MKLVARS_VERBOSE=verbose;
+        elif [ "$1" = "--help" ]                ; then mkl_help; return 254 2>/dev/null || exit 254;
         fi
         shift;
     done
 fi
 
-if [ -z "${__mkl_help}" ] ; then
-    __mkl_lib_dir="${MKLROOT}/lib"
-    __cpath="${MKLROOT}/include"
-    __mkl_bin_dir="${MKLROOT}/bin"
+# Set environment variables
+export MKLROOT=$(dirname -- "${__mkl_tmp_SCRIPT_DIR}")  # grandparent directory of this script
+__mkl_tmp_CPATH="${MKLROOT}/include"
+if [ "${__mkl_tmp_MOD}" = "${__mkl_tmp_MOD_NAME}" ] ; then
+    __mkl_tmp_CPATH="${MKLROOT}/include/mkl/intel64/${__mkl_tmp_LP64_ILP64}:${__mkl_tmp_CPATH}"
+fi
+export CPATH=$(prepend_path "${__mkl_tmp_CPATH}" "${CPATH:-}")
+export LD_LIBRARY_PATH=$(prepend_path "${MKLROOT}/lib" "${LD_LIBRARY_PATH:-}")
+export LIBRARY_PATH=$(prepend_path "${MKLROOT}/lib" "${LIBRARY_PATH:-}")
+export PATH=$(prepend_path "${MKLROOT}/bin" "${PATH:-}")
+export PKG_CONFIG_PATH=$(prepend_path "${MKLROOT}/lib/pkgconfig" "${PKG_CONFIG_PATH:-}")
+export CMAKE_PREFIX_PATH=$(prepend_path "${MKLROOT}/lib/cmake" "${CMAKE_PREFIX_PATH:-}")
 
-    __ld_library_path="${__mkl_lib_dir}/${__mkl_tmp_TARGET_ARCH}"
-    __library_path="${__mkl_lib_dir}/${__mkl_tmp_TARGET_ARCH}"
-    __path="${__mkl_bin_dir}/${__mkl_tmp_TARGET_ARCH}"
-    __nlspath="${__mkl_lib_dir}/${__mkl_tmp_TARGET_ARCH}/locale/%l_%t/%N"
-
-    if [ "${__mkl_tmp_MOD}" = "${__mkl_tmp_MOD_NAME}" ] ; then
-        if [ "${__mkl_tmp_TARGET_ARCH}" = "ia32" ] ; then
-            __mkl_tmp_LP64_ILP64=
-        else
-            if [ -z "$__mkl_tmp_LP64_ILP64" ] ; then
-                __mkl_tmp_LP64_ILP64=lp64
-            fi
-        fi
-        __cpath="${MKLROOT}/include/${__mkl_tmp_TARGET_ARCH}/${__mkl_tmp_LP64_ILP64}:${__cpath}"
-    fi
-
-    export LD_LIBRARY_PATH="${__ld_library_path}${LD_LIBRARY_PATH+:${LD_LIBRARY_PATH}}"
-    export LIBRARY_PATH="${__library_path}${LIBRARY_PATH+:${LIBRARY_PATH}}"
-    export NLSPATH="${__nlspath}${NLSPATH+:${NLSPATH}}"
-    export CPATH="${__cpath}${CPATH+:${CPATH}}"
-    export PATH="${__path}${PATH+:${PATH}}"
-    export PKG_CONFIG_PATH="${MKLROOT}/lib/pkgconfig${PKG_CONFIG_PATH+:${PKG_CONFIG_PATH}}"
-
-    if [ "${__mkl_tmp_MKLVARS_VERBOSE}" = "verbose" ] ; then
-        echo MKLROOT=${MKLROOT}
-        echo LD_LIBRARY_PATH=${LD_LIBRARY_PATH}
-        echo LIBRARY_PATH=${LIBRARY_PATH}
-        echo NLSPATH=${NLSPATH}
-        echo CPATH=${CPATH}
-        echo PKG_CONFIG_PATH=${PKG_CONFIG_PATH}
-    fi
+if [ "${__mkl_tmp_MKLVARS_VERBOSE}" = "verbose" ] ; then
+    echo MKLROOT=${MKLROOT}
+    echo CPATH=${CPATH}
+    echo LD_LIBRARY_PATH=${LD_LIBRARY_PATH}
+    echo LIBRARY_PATH=${LIBRARY_PATH}
+    echo PATH=${PATH}
+    echo PKG_CONFIG_PATH=${PKG_CONFIG_PATH}
+    echo CMAKE_PREFIX_PATH=${CMAKE_PREFIX_PATH}
 fi
