@@ -77,31 +77,33 @@
 *
 *  In what follows the symbol ' means transposition of object preceding the symbol.
 *
-*  The test performs the following operations :
+*  Using notation A = (L + D + U) for provided matrix elements and I for identity
+*  matrix, the test performs the following operations :
 *
-*       1. The code computes (L+D)'*S = F using mkl_sparse_d_mm  where S is a known 5 by 2
-*          matrix and then the code solves the system (L+D)'*X = F with the help of
-*          mkl_sparse_d_trsm. It's evident that X should be equal to S.
+*       1. The example computes (L+D)'*x_m_original = y_m using mkl_sparse_d_mm  where x_m_original is a known 5 by 2
+*          matrix and then the example solves the system (L+D)'*x_m_calculated = y_m with the help of
+*          mkl_sparse_d_trsm. It's evident that x_m_calculated should be equal to x_m_original.
 *
-*       2. The code computes (U+I)*s = F using mkl_sparse_d_mv where s is a vector
-*          and then the code calls mkl_sparse_d_trsv which  solves the system
-*          (U+I)*x = F with the single right hand side. It's evident that x
-*          should be equal to s.
+*       2. The example computes (U+I)*x_v_calculated = y_v using mkl_sparse_d_mv where x_v_calculated is a vector
+*          and then the example calls mkl_sparse_d_trsv which  solves the system
+*          (U+I)*x_v_calculated = y_v with the single right hand side. It's evident that x_v_calculated
+*          should be equal to x_v_original.
 *
-*       3. The code computes D*s = f using mkl_sparse_d_mv  where s is a vector
-*          and then the code solves the system D*x = f with the single right hand side.
-*          It's evident that s should be equal to s.
+*       3. The example computes D*x_v_original = y_v using mkl_sparse_d_mv  where x_v_original is a vector
+*          and then the example solves the system D*x_v_calculated = y_v with the single right hand side.
+*          It's evident that x_v_calculated should be equal to x_v_original.
 *
-*       4. The next step is the computation (L+D+L') s = f using mkl_sparse_d_mv
-*          where s is a vector. It is easy to see that L+D+L' is a symmetric matrix.
+*       4. The next step is the computation (L+D+L') x_v_original = y_v using mkl_sparse_d_mv
+*          where x_v_original is a vector. It is easy to see that L+D+L' is a symmetric matrix.
 *
-*       5. The next step is the computation A'* s = f using mkl_sparse_mv where s
+*       5. The next step is the computation A'* x_v_original = y_v using mkl_sparse_mv where x_v_original
 *          is a vector.
 *
 * The code given below uses only one sparse representation for the all operations.
 ********************************************************************************
 */
 #include <stdio.h>
+#include <math.h>
 #include "mkl_types.h"
 #include "mkl_spblas.h"
 
@@ -111,6 +113,8 @@
 #define INT_PRINT_FORMAT "%d"
 #endif
 
+#define ALMOST_EQUAL(a, b) (fabs((a)-(b)) < 1e-10 ? 1 : 0)
+
 int main () {
 //*******************************************************************************
 //     Definition arrays for sparse representation of the matrix A in
@@ -123,33 +127,33 @@ int main () {
 
     MKL_INT colIndex[M+1] = {   0,               3,              6,              9,        11,       13};
     MKL_INT rows[NNZ]     = {   0,   1,    3,    0,   1,   4,    0,   2,   3,    2,   3,    2,   4};
-    double values[NNZ]    = { 1.0 -2.0, -4.0, -1.0, 5.0, 8.0, -3.0, 4.0, 2.0,  6.0, 7.0, 4.0, -5.0};
+    double values[NNZ]    = { 1.0, -2.0, -4.0, -1.0, 5.0, 8.0, -3.0, 4.0, 2.0,  6.0, 7.0, 4.0, -5.0};
 
 //*******************************************************************************
 //    Declaration of local variables :
 //*******************************************************************************
 
-    double   sol[M*NRHS]   = {1.0, 5.0,
-                              1.0, 4.0,
-                              1.0, 3.0,
-                              1.0, 2.0,
-                              1.0, 1.0};
+    double   x_m_original[M*NRHS]   = {1.0, 5.0,
+                                       1.0, 4.0,
+                                       1.0, 3.0,
+                                       1.0, 2.0,
+                                       1.0, 1.0};
 
-    double   rhs[M*NRHS]   = {0.0, 0.0,
-                              0.0, 0.0,
-                              0.0, 0.0,
-                              0.0, 0.0,
-                              0.0, 0.0};
+    double   y_m[M*NRHS]            = {0.0, 0.0,
+                                       0.0, 0.0,
+                                       0.0, 0.0,
+                                       0.0, 0.0,
+                                       0.0, 0.0};
 
-    double   tmp[M*NRHS]   = {0.0, 0.0,
-                              0.0, 0.0,
-                              0.0, 0.0,
-                              0.0, 0.0,
-                              0.0, 0.0};
+    double   x_m_calculated[M*NRHS] = {0.0, 0.0,
+                                       0.0, 0.0,
+                                       0.0, 0.0,
+                                       0.0, 0.0,
+                                       0.0, 0.0};
 
-    double      sol_vec[M]  = {1.0, 1.0, 1.0, 1.0, 1.0};
-    double      rhs_vec[M]  = {0.0, 0.0, 0.0, 0.0, 0.0};
-    double      temp_vec[M] = {0.0, 0.0, 0.0, 0.0, 0.0};
+    double      x_v_original[M]   = {1.0, 1.0, 1.0, 1.0, 1.0};
+    double      y_v[M]            = {0.0, 0.0, 0.0, 0.0, 0.0};
+    double      x_v_calculated[M] = {0.0, 0.0, 0.0, 0.0, 0.0};
 
     double      alpha = 1.0, beta = 0.0;
     MKL_INT     i, j;
@@ -291,9 +295,9 @@ int main () {
     }
 
 //*******************************************************************************
-//    Task 1.  Obtain Triangular matrix-matrix multiply (L+D)' *sol --> rhs
-//    and solve triangular system   (L+D)' *tmp = rhs with multiple right
-//    hand sides. Array tmp must be equal to the array sol
+//    Task 1.  Obtain Triangular matrix-matrix multiply (L+D)' *x_m_original --> y_m
+//    and solve triangular system   (L+D)' *x_m_calculated = y_m with multiple right
+//    hand sides. Array x_m_calculated must be equal to the array x_m_original
 //*******************************************************************************
     printf("                                  \n");
     printf("   TASK 1:                        \n");
@@ -305,7 +309,7 @@ int main () {
     printf("   Input matrix                   \n");
     for (i = 0; i < m; i++) {
         for (j = 0; j < nrhs; j++) {
-            printf("%7.1f", sol[i*nrhs+j]);
+            printf("%7.1f", x_m_original[i*nrhs+j]);
         };
         printf("\n");
     };
@@ -315,8 +319,8 @@ int main () {
     descrA.diag = SPARSE_DIAG_NON_UNIT;
 
     status = mkl_sparse_d_mm( SPARSE_OPERATION_TRANSPOSE, alpha, cscA, descrA,
-                              SPARSE_LAYOUT_ROW_MAJOR, sol, nrhs, nrhs, beta,
-                              rhs, nrhs);
+                              SPARSE_LAYOUT_ROW_MAJOR, x_m_original, nrhs, nrhs, beta,
+                              y_m, nrhs);
     if (status != SPARSE_STATUS_SUCCESS) {
         printf(" Error in Task 1 mkl_sparse_d_mm: %d \n", status);
         exit_status = 1;
@@ -328,7 +332,7 @@ int main () {
     printf("   WITH TRIANGULAR MATRIX          \n");
     for (i = 0; i < m; i++) {
         for (j = 0; j < nrhs; j++) {
-            printf("%7.1f", rhs[i*nrhs+j]);
+            printf("%7.1f", y_m[i*nrhs+j]);
         };
         printf("\n");
     };
@@ -339,7 +343,7 @@ int main () {
     printf("   right hand side           \n");
     printf("                             \n");
 
-    status =  mkl_sparse_d_trsm( SPARSE_OPERATION_TRANSPOSE, alpha, cscA, descrA, SPARSE_LAYOUT_ROW_MAJOR, rhs, nrhs, nrhs, tmp, nrhs);
+    status =  mkl_sparse_d_trsm( SPARSE_OPERATION_TRANSPOSE, alpha, cscA, descrA, SPARSE_LAYOUT_ROW_MAJOR, y_m, nrhs, nrhs, x_m_calculated, nrhs);
     if (status != SPARSE_STATUS_SUCCESS) {
         printf(" Error in Task 1 mkl_sparse_d_trsm: %d \n", status);
         exit_status = 1;
@@ -351,16 +355,28 @@ int main () {
     printf("   WITH TRIANGULAR MATRIX            \n");
     for (i = 0; i < m; i++) {
         for (j = 0; j < nrhs; j++) {
-            printf("%7.1f", tmp[i*nrhs+j]);
+            printf("%7.1f", x_m_calculated[i*nrhs+j]);
         };
         printf("\n");
     };
+
+    printf( "   Validating output data\n" );
+    printf( "   against input data\n" );
+    for (i = 0; i < m*nrhs; i++) {
+        if (!ALMOST_EQUAL(x_m_original[i], x_m_calculated[i])) {
+            printf( " Error in Task 5: output data is not equal to" );
+            printf( " input data \n" );
+            exit_status = 1;
+            goto exit;
+        };
+    };
+    printf( "   Done \n" );
     printf("-----------------------------------------------\n");
 
 //*******************************************************************************
-//    Task 2.    Obtain Triangular matrix-vector multiply (U+I) *sol --> rhs
-//    and solve triangular system   (U+I) *tmp = rhs with single right hand sides
-//    Array tmp must be equal to the array sol
+//    Task 2.    Obtain Triangular matrix-vector multiply (U+I) *x_v_original --> y_v
+//    and solve triangular system   (U+I) *x_v_calculated = y_v with single right hand sides
+//    Array x_v_calculated must be equal to the array x_v_original
 //*******************************************************************************
     printf("                                     \n");
     printf("   TASK 2:                           \n");
@@ -370,14 +386,14 @@ int main () {
     printf("     SPARSE_OPERATION_NON_TRANSPOSE  \n" );
     printf("   Input vector                      \n");
     for (i = 0; i < m; i++) {
-        printf("%7.1f\n", sol_vec[i]);
+        printf("%7.1f\n", x_v_original[i]);
     };
 
     descrA.type = SPARSE_MATRIX_TYPE_TRIANGULAR;
     descrA.mode = SPARSE_FILL_MODE_UPPER;
     descrA.diag = SPARSE_DIAG_UNIT;
 
-    status = mkl_sparse_d_mv( SPARSE_OPERATION_NON_TRANSPOSE, alpha, cscA, descrA, sol_vec, beta, rhs_vec);
+    status = mkl_sparse_d_mv( SPARSE_OPERATION_NON_TRANSPOSE, alpha, cscA, descrA, x_v_original, beta, y_v);
     if (status != SPARSE_STATUS_SUCCESS) {
         printf(" Error in Task 2 mkl_sparse_d_mv: %d \n", status);
         exit_status = 1;
@@ -388,14 +404,14 @@ int main () {
     printf("   OUTPUT DATA FOR mkl_sparse_d_mv \n");
     printf("   WITH TRIANGULAR MATRIX          \n");
     for (i = 0; i < m; i++) {
-        printf("%7.1f\n", rhs_vec[i]);
+        printf("%7.1f\n", y_v[i]);
     };
     printf("-----------------------------------------------\n");
     printf("   Solve triangular system   \n");
     printf("   with obtained             \n");
     printf("   right hand side           \n");
 
-    status =  mkl_sparse_d_trsv( SPARSE_OPERATION_NON_TRANSPOSE, alpha, cscA, descrA, rhs_vec, temp_vec);
+    status =  mkl_sparse_d_trsv( SPARSE_OPERATION_NON_TRANSPOSE, alpha, cscA, descrA, y_v, x_v_calculated);
     if (status != SPARSE_STATUS_SUCCESS) {
         printf(" Error in Task 2 mkl_sparse_d_trsv: %d \n", status);
         exit_status = 1;
@@ -406,14 +422,37 @@ int main () {
     printf("   OUTPUT DATA FOR mkl_sparse_d_trsv \n");
     printf("   WITH TRIANGULAR MATRIX            \n");
     for (i = 0; i < m; i++) {
-        printf("%7.1f\n", temp_vec[i]);
+        printf("%7.1f\n", x_v_calculated[i]);
     };
+
+    printf( "   Validating output data\n" );
+    printf( "   against input data\n" );
+    for (i = 0; i < m; i++) {
+        if (!ALMOST_EQUAL(x_v_original[i], x_v_calculated[i])) {
+            printf( " Error in Task 2: output data is not equal to" );
+            printf( " input data \n" );
+            exit_status = 1;
+            goto exit;
+        }
+    };
+
+    printf( "   Validating output data\n" );
+    printf( "   against input data\n" );
+    for (i = 0; i < m; i++) {
+        if (!ALMOST_EQUAL(x_v_original[i], x_v_calculated[i])) {
+            printf( " Error in Task 2: output data is not equal to" );
+            printf( " input data \n" );
+            exit_status = 1;
+            goto exit;
+        }
+    };
+    printf( "   Done \n" );
     printf("-----------------------------------------------\n");
 
 //*******************************************************************************
-//    Task 3.  Obtain Diagonal matrix-vector multiply D *sol --> rhs
-//    and solve triangular system   D *temp = rhs with single right hand side
-//    Array temp must be equal to the array sol
+//    Task 3.  Obtain Diagonal matrix-vector multiply D *x_v_original --> y_v
+//    and solve triangular system   D *x_v_calculated = y_v with single right hand side
+//    Array x_v_calculated must be equal to the array x_v_original
 //*******************************************************************************
     printf("                                    \n");
     printf("   TASK 3:                          \n");
@@ -423,13 +462,13 @@ int main () {
     printf("     SPARSE_OPERATION_NON_TRANSPOSE \n" );
     printf("   Input vector                     \n");
     for (i = 0; i < m; i++) {
-        printf("%7.1f\n", sol_vec[i]);
+        printf("%7.1f\n", x_v_original[i]);
     };
 
     descrA.type = SPARSE_MATRIX_TYPE_DIAGONAL;
     descrA.diag = SPARSE_DIAG_NON_UNIT;
 
-    status = mkl_sparse_d_mv( SPARSE_OPERATION_NON_TRANSPOSE, alpha, cscA, descrA, sol_vec, beta, rhs_vec);
+    status = mkl_sparse_d_mv( SPARSE_OPERATION_NON_TRANSPOSE, alpha, cscA, descrA, x_v_original, beta, y_v);
     if (status != SPARSE_STATUS_SUCCESS) {
         printf(" Error in Task 3 mkl_sparse_d_mv: %d \n", status);
         exit_status = 1;
@@ -440,14 +479,14 @@ int main () {
     printf("   OUTPUT DATA FOR mkl_sparse_d_mv \n");
     printf("   WITH DIAGONAL MATRIX            \n");
     for (i = 0; i < m; i++) {
-        printf("%7.1f\n", rhs_vec[i]);
+        printf("%7.1f\n", y_v[i]);
     };
     printf("-----------------------------------------------\n");
     printf("   Multiply by inverse      \n");
     printf("   matrix with the help     \n");
     printf("   of MKL_SPARSE_D_TRSV            \n");
 
-    status =  mkl_sparse_d_trsv( SPARSE_OPERATION_NON_TRANSPOSE, alpha, cscA, descrA, rhs_vec, temp_vec);
+    status =  mkl_sparse_d_trsv( SPARSE_OPERATION_NON_TRANSPOSE, alpha, cscA, descrA, y_v, x_v_calculated);
     if (status != SPARSE_STATUS_SUCCESS) {
         printf(" Error in Task 3  mkl_sparse_d_trsv: %d \n", status);
         exit_status = 1;
@@ -458,12 +497,35 @@ int main () {
     printf("   OUTPUT DATA FOR mkl_sparse_d_trsv \n");
     printf("   WITH DIAGONAL MATRIX              \n");
     for (i = 0; i < m; i++) {
-        printf("%7.1f\n", temp_vec[i]);
+        printf("%7.1f\n", x_v_calculated[i]);
     };
+
+    printf( "   Validating output data\n" );
+    printf( "   against input data\n" );
+    for (i = 0; i < m; i++) {
+        if (!ALMOST_EQUAL(x_v_original[i], x_v_calculated[i])) {
+            printf( " Error in Task 3: output data is not equal to" );
+            printf( " input data \n" );
+            exit_status = 1;
+            goto exit;
+        }
+    };
+
+    printf( "   Validating output data\n" );
+    printf( "   against input data\n" );
+    for (i = 0; i < m; i++) {
+        if (!ALMOST_EQUAL(x_v_original[i], x_v_calculated[i])) {
+            printf( " Error in Task 3: output data is not equal to" );
+            printf( " input data \n" );
+            exit_status = 1;
+            goto exit;
+        }
+    };
+    printf( "   Done \n" );
     printf("-----------------------------------------------\n");
 
 //*******************************************************************************
-//    Task 4.  Obtain Symmetric matrix-vector multiply (L+D+L')*sol --> rhs
+//    Task 4.  Obtain Symmetric matrix-vector multiply (L+D+L')*x_v_original --> y_v
 //    with the help of MKL_SPARSE_D_MV
 //
 //*******************************************************************************
@@ -475,14 +537,14 @@ int main () {
     printf("     SPARSE_OPERATION_NON_TRANSPOSE \n" );
     printf("   Input vector                     \n");
     for (i = 0; i < m; i++) {
-        printf("%7.1f\n", sol_vec[i]);
+        printf("%7.1f\n", x_v_original[i]);
     };
 
     descrA.type = SPARSE_MATRIX_TYPE_SYMMETRIC;
     descrA.mode = SPARSE_FILL_MODE_LOWER;
     descrA.diag = SPARSE_DIAG_NON_UNIT;
 
-    status = mkl_sparse_d_mv( SPARSE_OPERATION_NON_TRANSPOSE, alpha, cscA, descrA, sol_vec, beta, rhs_vec);
+    status = mkl_sparse_d_mv( SPARSE_OPERATION_NON_TRANSPOSE, alpha, cscA, descrA, x_v_original, beta, y_v);
     if (status != SPARSE_STATUS_SUCCESS) {
         printf(" Error in Task 4 mkl_sparse_d_mv: %d \n", status);
         exit_status = 1;
@@ -493,12 +555,12 @@ int main () {
     printf("   OUTPUT DATA FOR mkl_sparse_d_mv \n");
     printf("   WITH SYMMETRIC LOWER MATRIX     \n");
     for (i = 0; i < m; i++) {
-        printf("%7.1f\n", rhs_vec[i]);
+        printf("%7.1f\n", y_v[i]);
     };
     printf("-----------------------------------------------\n");
 
 //*******************************************************************************
-//    Task 5. Obtain General matrix-vector multiply A'*sol --> rhs with
+//    Task 5. Obtain General matrix-vector multiply A'*x_v_original --> y_v with
 //    the help of MKL_SPARSE_D_MV
 //
 //*******************************************************************************
@@ -510,13 +572,13 @@ int main () {
     printf("     SPARSE_OPERATION_TRANSPOSE  \n" );
     printf("   Input vector                  \n");
     for (i = 0; i < m; i++) {
-        printf("%7.1f\n", sol_vec[i]);
+        printf("%7.1f\n", x_v_original[i]);
     };
 
     descrA.type = SPARSE_MATRIX_TYPE_GENERAL;
     descrA.diag = SPARSE_DIAG_NON_UNIT;
 
-    status = mkl_sparse_d_mv( SPARSE_OPERATION_TRANSPOSE, alpha, cscA, descrA, sol_vec, beta, rhs_vec);
+    status = mkl_sparse_d_mv( SPARSE_OPERATION_TRANSPOSE, alpha, cscA, descrA, x_v_original, beta, y_v);
     if (status != SPARSE_STATUS_SUCCESS) {
         printf(" Error in Task 5 mkl_sparse_d_mv: %d \n", status);
         exit_status = 1;
@@ -527,7 +589,7 @@ int main () {
     printf("   OUTPUT DATA FOR mkl_sparse_d_mv \n");
     printf("   WITH GENERAL MATRIX             \n");
     for (i = 0; i < m; i++) {
-        printf("%7.1f\n", rhs_vec[i]);
+        printf("%7.1f\n", y_v[i]);
     };
     printf("-----------------------------------------------\n");
 
