@@ -115,7 +115,7 @@ int main (void)
     for(k=1;k<n;k++)
         for(j=1;j<n;j++)
             for(i=1;i<n;i++)
-                rhs[k*(n+1)*(n+1)+j*(n+1)+i]=k*k+i*i+j*j;
+                rhs[k*(n+1)*(n+1)+j*(n+1)+i]=(double)k*k+i*i+j*j;
 
     /* Initialize zero dirichlet boundary condition */
     for (k=0;k<(n+1)*(n+1);k++) 
@@ -272,7 +272,6 @@ int main (void)
             return finalize(tmp, solution, dpar_helm, rhs, bd_ax, 
                             bd_bx, bd_ay, bd_by, bd_az, bd_bz, 
                             xhandle, yhandle, ipar_helm, 1);
-            break;
         }
     } while (rci_request != 0);
 
